__version__ = "0.1.0"
__title__ = "pytest-snail"
__description__ = "Plugin for adding a marker to slow running tests. 🐌"
__url__ = "https://github.com/hackebrot/pytest-snail"

__author__ = "Raphael Pierzina"
__email__ = "raphael@hackebrot.de"

__license__ = "MIT License"
__copyright__ = "Copyright (c) 2019 Raphael Pierzina"



"""Plugin for adding a marker to slow running tests."""

from collections import defaultdict

import pytest


class Snail:
    """Plugin for adding a marker to slow running tests."""

    def __init__(self, config):
        """Initialize the plugin and load test durations from the cache."""
        self.config = config
        self.durations = defaultdict(dict)
        self.durations.update(self.config.cache.get("cache/snail", defaultdict(dict)))

    def pytest_runtest_logreport(self, report):
        """Keep track of test durations for test items by test phase."""
        self.durations[report.nodeid][report.when] = report.duration

    @pytest.mark.tryfirst
    def pytest_collection_modifyitems(self, session, config, items):
        """Add the snail marker to slow running tests."""
        for item in items:
            duration = sum(self.durations[item.nodeid].values())
            if duration >= self.config.option.snail:
                item.add_marker(pytest.mark.snail)

    def pytest_sessionfinish(self, session):
        """Write test durations to the cache."""
        cached_durations = self.config.cache.get("cache/snail", defaultdict(dict))
        cached_durations.update(self.durations)
        self.config.cache.set("cache/snail", cached_durations)

    def pytest_configure(self, config):
        """Register and document the snail marker."""
        config.addinivalue_line("markers", "snail: marker for slow running tests")


def pytest_addoption(parser):
    """Add the --snail CLI option."""
    group = parser.getgroup("snail")
    group.addoption(
        "--snail",
        action="store",
        type=float,
        default=5.0,
        metavar="DURATION",
        help="tests running longer than this value will be marked",
    )


def pytest_configure(config):
    """Hook implementation that registers the plugin."""
    config.pluginmanager.register(Snail(config), "snail_plugin")


def pytest_unconfigure(config):
    """Hook implementation that unregisters the plugin."""
    config.pluginmanager.unregister("snail_plugin")



import pathlib
import setuptools


def read(*args):
    file_path = pathlib.Path(__file__).parent.joinpath(*args)
    return file_path.read_text("utf-8")


setuptools.setup(
    name="pytest-snail",
    version="0.1.0",
    author="Raphael Pierzina",
    author_email="raphael@hackebrot.de",
    maintainer="Raphael Pierzina",
    maintainer_email="raphael@hackebrot.de",
    license="MIT License",
    url="https://github.com/hackebrot/pytest-snail",
    project_urls={
        "Repository": "https://github.com/hackebrot/pytest-snail",
        "Issues": "https://github.com/hackebrot/pytest-snail/issues",
    },
    description="Plugin for adding a marker to slow running tests. 🐌",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages("src"),
    package_dir={"": "src"},
    include_package_data=True,
    zip_safe=False,
    python_requires=">=3.6",
    install_requires=["pytest>=5.0.1"],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Framework :: Pytest",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Testing",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: MIT License",
    ],
    entry_points={"pytest11": ["snail = pytest_snail.plugin"]},
    keywords=["pytest"],
)
